package com.qmth.teachcloud.common.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.boot.core.solar.config.SolarProperties;
import com.qmth.boot.core.solar.model.AppInfo;
import com.qmth.boot.core.solar.model.OrgInfo;
import com.qmth.boot.core.solar.service.SolarService;
import com.qmth.teachcloud.common.config.DictionaryConfig;
import com.qmth.teachcloud.common.contant.SpringContextHolder;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicSchool;
import com.qmth.teachcloud.common.entity.SysOrg;
import com.qmth.teachcloud.common.entity.TSAuth;
import com.qmth.teachcloud.common.enums.AuthEnum;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.UploadFileEnum;
import com.qmth.teachcloud.common.service.*;
import com.qmth.teachcloud.common.util.FileStoreUtil;
import org.apache.commons.codec.digest.DigestUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileInputStream;
import java.time.LocalDateTime;
import java.util.*;

/**
 * @Description: 授权信息service
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2022/5/17
 */
@Service
public class AuthInfoServiceImpl implements AuthInfoService {
    private final static Logger log = LoggerFactory.getLogger(AuthInfoServiceImpl.class);

    @Resource
    SolarProperties solarProperties;

    @Resource
    SolarService solarService;

    @Resource
    TSAuthService tsAuthService;

    @Resource
    CommonCacheService commonCacheService;

    @Resource
    BasicSchoolService basicSchoolService;

    @Resource
    FileStoreUtil fileStoreUtil;

    @Resource
    DictionaryConfig dictionaryConfig;

    @Resource
    AuthInfoService authInfoService;

    @Resource
    SysOrgService sysOrgService;

    /**
     * 授权信息初始化
     *
     * @return
     */
    @Override
    public AppInfo appInfoInit() {
        AppInfo appInfo = null;
        try {
            appInfo = solarService.getAppInfo();
            if (Objects.nonNull(appInfo) && Objects.nonNull(solarProperties)) {
                if (Objects.nonNull(solarProperties.getAccessKey())
                        && !Objects.equals(solarProperties.getAccessKey().trim(), "")
                        && Objects.nonNull(solarProperties.getAccessSecret())
                        && !Objects.equals(solarProperties.getAccessSecret().trim(), "")) {//在线激活
//                    this.authError("不支持在线激活，请联系系统管理员！");
                    log.error("不支持在线激活，请联系系统管理员！");
                    licenseForDb();
                } else if (Objects.nonNull(solarProperties.getLicense())
                        && !Objects.equals(solarProperties.getLicense().trim(), "")) {//离线激活
//                    this.authError("不支持离线激活，请联系系统管理员！");
                    log.error("不支持离线激活，请联系系统管理员！");
                    licenseForDb();
                }
            } else {
                licenseForDb();
            }
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
        }
        return appInfo;
    }

    /**
     * app是否过期
     *
     * @param code
     * @return
     */
    @Override
    public void appHasExpired(String code) {
        BasicSchool basicSchool = commonCacheService.schoolCache(code);
        if ((Objects.nonNull(basicSchool) && Objects.nonNull(basicSchool.getEnable()) && !basicSchool.getEnable())
                || (Objects.isNull(solarService.getAppInfo()) || solarService.getAppInfo().getControl().hasExpired())
                || (Objects.isNull(solarService.getAppControl()) || solarService.getAppControl().hasExpired())) {
            throw ExceptionResultEnum.AUTH_INFO_ERROR.exception();
        }
    }

    /**
     * 离线激活
     *
     * @param licenseData
     */
    @Override
    public void updateLicense(byte[] licenseData) throws Exception {
        AppInfo appInfo = solarService.update(licenseData);
        if (Objects.isNull(appInfo)) {
            throw ExceptionResultEnum.AUTH_INFO_ERROR.exception("激活失败");
        }
        authInfoService.saveAuthInfo(appInfo, AuthEnum.OFF_LINE, licenseData);
    }

    /**
     * 查询授权信息
     *
     * @return
     */
    @Override
    public Long selectAuthInfo() {
        Long expireTime = null;
        AppInfo appInfo = solarService.getAppInfo();
        if (Objects.nonNull(appInfo) && Objects.nonNull(appInfo.getControl())) {
            expireTime = Objects.nonNull(appInfo.getControl().getExpireTime()) ? appInfo.getControl().getExpireTime() : -1;
        }
        return expireTime;
    }

    /**
     * 保存鉴权信息
     *
     * @param appInfo
     * @param authEnum
     * @param file
     * @throws Exception
     */
    @Override
    @Transactional
    public void saveAuthInfo(AppInfo appInfo, AuthEnum authEnum, byte[] file) throws Exception {
        List<OrgInfo> orgInfoList = solarService.getOrgList();
        List<TSAuth> tsAuthList = null;
        Set<Long> orgIdsSet = null;
        Map<Long, Long> orgIdsMap = null;
        Set<BasicSchool> basicSchoolSet = null;
        Set<SysOrg> sysOrgSet = null;
        if (!CollectionUtils.isEmpty(orgInfoList)) {
            tsAuthList = new ArrayList<>();
            orgIdsSet = new HashSet<>();
            basicSchoolSet = new HashSet<>();
            sysOrgSet = new HashSet<>();
            orgIdsMap = new HashMap<>();
        }
        boolean oss = dictionaryConfig.sysDomain().isOss();
        for (OrgInfo o : orgInfoList) {
            orgIdsSet.add(o.getId());
            orgIdsMap.put(o.getId(), o.getId());
            if (authEnum == AuthEnum.OFF_LINE) {
                if (Objects.isNull(file)) {
                    tsAuthList.add(new TSAuth(o.getId(), solarProperties.getLicense(), authEnum, appInfo.getControl().getExpireTime()));
                } else {
                    tsAuthList.add(new TSAuth(o.getId(), file, authEnum, appInfo.getControl().getExpireTime()));
                }
            } else {
                tsAuthList.add(new TSAuth(o.getId(), solarProperties.getAccessKey(), solarProperties.getAccessSecret(), authEnum, appInfo.getControl().getExpireTime()));
            }
            QueryWrapper<BasicSchool> basicSchoolQueryWrapper = new QueryWrapper<>();
            basicSchoolQueryWrapper.lambda().eq(BasicSchool::getCode, o.getCode());
            BasicSchool basicSchool = basicSchoolService.getOne(basicSchoolQueryWrapper);
            if (Objects.isNull(basicSchool)) {//不存在则创建学校
                basicSchool = new BasicSchool(o.getId(), o.getCode(), o.getName(), o.getAccessKey(), o.getAccessSecret());
                if (Objects.nonNull(o.getLogo()) && (!o.getLogo().startsWith("https:") || !o.getLogo().startsWith("http"))) {
                    String filePath = dictionaryConfig.fssLocalFileDomain().getConfig() + File.separator + SystemConstant.getUuid() + ".jpg";
                    File logoFile = new File(filePath);
                    if (!logoFile.getParentFile().exists()) {
                        // 不存在则创建父目录及子文件
                        logoFile.getParentFile().mkdirs();
                        logoFile.createNewFile();
                    }
                    SystemConstant.base64ToImage(o.getLogo(), filePath);

                    if (oss) {
                        LocalDateTime nowTime = LocalDateTime.now();
                        StringJoiner stringJoiner = new StringJoiner("");
                        stringJoiner.add(UploadFileEnum.FILE.name().toLowerCase()).add(File.separator)
                                .add(String.valueOf(nowTime.getYear())).add(File.separator)
                                .add(String.format("%02d", nowTime.getMonthValue())).add(File.separator)
                                .add(String.format("%02d", nowTime.getDayOfMonth())).add(File.separator)
                                .add(SystemConstant.getUuid()).add(".jpg");
                        fileStoreUtil.ossUpload(stringJoiner.toString(), logoFile, DigestUtils.md5Hex(new FileInputStream(logoFile)), UploadFileEnum.FILE.getFssType());
                        logoFile.delete();
                        basicSchool.setLogo(fileStoreUtil.getPrivateUrl(stringJoiner.toString(), UploadFileEnum.FILE.getFssType()));
                    } else {
                        basicSchool.setLogo(filePath);
                    }
                } else {
                    basicSchool.setLogo(o.getLogo());
                }
                basicSchoolSet.add(basicSchool);
            }

            QueryWrapper<SysOrg> sysOrgQueryWrapper = new QueryWrapper<>();
            sysOrgQueryWrapper.lambda().eq(SysOrg::getSchoolId, basicSchool.getId());
            int count = sysOrgService.count(sysOrgQueryWrapper);
            if (count == 0) {
                sysOrgSet.add(new SysOrg(basicSchool.getId(), basicSchool.getName()));
            }
        }

        if (!CollectionUtils.isEmpty(tsAuthList) && !CollectionUtils.isEmpty(orgIdsSet)) {
            tsAuthService.remove(new QueryWrapper<TSAuth>().lambda().ge(TSAuth::getId, 0L));
            tsAuthService.saveOrUpdateBatch(tsAuthList);

            if (!CollectionUtils.isEmpty(basicSchoolSet)) {
                basicSchoolService.saveOrUpdateBatch(basicSchoolSet);
                commonCacheService.removeSchoolIdCache();
                commonCacheService.removeSchoolCodeCache();
            }

            AuthInfoService authInfoService = SpringContextHolder.getBean(AuthInfoService.class);
            authInfoService.updateSchoolEnable(orgIdsMap);

            if (!CollectionUtils.isEmpty(sysOrgSet)) {
                sysOrgService.saveOrUpdateBatch(sysOrgSet);
                commonCacheService.removeOrgCache();
            }
        }
    }

//    /**
//     * 激活失败
//     *
//     * @param message
//     */
//    protected void authError(String message) {
//        List<OrgInfo> orgInfoList = solarService.getOrgList();
//        if (!CollectionUtils.isEmpty(orgInfoList)) {
//            Map<Long, Long> orgIdsMap = orgInfoList.stream().collect(Collectors.toMap(m -> m.getId(), m -> m.getId()));
//            this.updateSchoolEnable(orgIdsMap);
//        }
//        throw ExceptionResultEnum.AUTH_INFO_ERROR.exception(message);
//    }

    /**
     * 更新学校启用/禁用
     *
     * @param orgIdsMap
     */
    public void updateSchoolEnable(Map<Long, Long> orgIdsMap) {
        List<BasicSchool> basicSchoolList = basicSchoolService.list();
        if (!CollectionUtils.isEmpty(basicSchoolList)) {
            for (BasicSchool b : basicSchoolList) {
                if (orgIdsMap.containsKey(b.getId())) {
                    b.setEnable(true);
                } else {
                    b.setEnable(false);
                }
            }
            basicSchoolService.saveOrUpdateBatch(basicSchoolList);
            commonCacheService.removeSchoolIdCache();
            commonCacheService.removeSchoolCodeCache();
        }
    }

    /**
     * 强制从db更新激活信息
     *
     * @throws Exception
     */
    public void licenseForDb() throws Exception {
        QueryWrapper<TSAuth> tsAuthQueryWrapper = new QueryWrapper<>();
        tsAuthQueryWrapper.lambda().isNotNull(TSAuth::getFile);
        List<TSAuth> tsAuthList = tsAuthService.list(tsAuthQueryWrapper);
        if (!CollectionUtils.isEmpty(tsAuthList)) {
            for (TSAuth t : tsAuthList) {
                AppInfo appInfo = solarService.update(t.getFile());
                authInfoService.saveAuthInfo(appInfo, AuthEnum.OFF_LINE, t.getFile());
            }
        }
    }
}
